@@ -508,6 +508,7 @@ module Sequel
     #                            argument.
     #            :implicit_qualifier :: The name to use for qualifying implicit conditions.  By default,
     #                                   the last joined or primary table is used.
+    #            :join_using :: Force the using of JOIN USING, even if +expr+ is not an array of symbols.
     #            :reset_implicit_qualifier :: Can set to false to ignore this join when future joins determine qualifier
     #                                         for implicit conditions.
     #            :qualify :: Can be set to false to not do any implicit qualification.  Can be set
@@ -541,7 +542,7 @@ module Sequel
         return s.join_table(type, ds, expr, options, &block)
       end
 
-      using_join = expr.is_a?(Array) && !expr.empty? && expr.all?{|x| x.is_a?(Symbol)}
+      using_join = options[:join_using] || (expr.is_a?(Array) && !expr.empty? && expr.all?{|x| x.is_a?(Symbol)})
       if using_join && !supports_join_using?
         h = {}
         expr.each{|e| h[e] = e}
@@ -616,7 +617,7 @@ module Sequel
     UNCONDITIONED_JOIN_TYPES.each do |jtype|
       class_eval(<<-END, __FILE__, __LINE__+1)
         def #{jtype}_join(table, opts=Sequel::OPTS)
-          raise(Sequel::Error, '#{jtype}_join does not accept join table blocks') if block_given?
+          raise(Sequel::Error, '#{jtype}_join does not accept join table blocks') if defined?(yield)
           raise(Sequel::Error, '#{jtype}_join 2nd argument should be an options hash, not conditions') unless opts.is_a?(Hash)
           join_table(:#{jtype}, table, nil, opts)
         end
@@ -677,6 +678,56 @@ module Sequel
       clone(:lock => style)
     end
     
+    # Return a dataset with a WHEN MATCHED THEN DELETE clause added to the
+    # MERGE statement.  If a block is passed, treat it as a virtual row and
+    # use it as additional conditions for the match.
+    #
+    #   merge_delete
+    #   # WHEN MATCHED THEN DELETE
+    #
+    #   merge_delete{a > 30}
+    #   # WHEN MATCHED AND (a > 30) THEN DELETE
+    def merge_delete(&block)
+      _merge_when(:type=>:delete, &block)
+    end
+
+    # Return a dataset with a WHEN NOT MATCHED THEN INSERT clause added to the
+    # MERGE statement.  If a block is passed, treat it as a virtual row and
+    # use it as additional conditions for the match.
+    #
+    # The arguments provided can be any arguments that would be accepted by
+    # #insert.
+    #
+    #   merge_insert(i1: :i2, a: Sequel[:b]+11)
+    #   # WHEN NOT MATCHED THEN INSERT (i1, a) VALUES (i2, (b + 11))
+    #
+    #   merge_insert(:i2, Sequel[:b]+11){a > 30}
+    #   # WHEN NOT MATCHED AND (a > 30) THEN INSERT VALUES (i2, (b + 11))
+    def merge_insert(*values, &block)
+      _merge_when(:type=>:insert, :values=>values, &block)
+    end
+    
+    # Return a dataset with a WHEN MATCHED THEN UPDATE clause added to the
+    # MERGE statement.  If a block is passed, treat it as a virtual row and
+    # use it as additional conditions for the match.
+    #
+    #   merge_update(i1: Sequel[:i1]+:i2+10, a: Sequel[:a]+:b+20)
+    #   # WHEN MATCHED THEN UPDATE SET i1 = (i1 + i2 + 10), a = (a + b + 20)
+    #
+    #   merge_update(i1: :i2){a > 30}
+    #   # WHEN MATCHED AND (a > 30) THEN UPDATE SET i1 = i2
+    def merge_update(values, &block)
+      _merge_when(:type=>:update, :values=>values, &block)
+    end
+
+    # Return a dataset with the source and join condition to use for the MERGE statement.
+    #
+    #   merge_using(:m2, i1: :i2)
+    #   # USING m2 ON (i1 = i2)
+    def merge_using(source, join_condition)
+      clone(:merge_using => [source, join_condition].freeze)
+    end
+
     # Returns a cloned dataset without a row_proc.
     #
     #   ds = DB[:items].with_row_proc(:invert.to_proc)
@@ -1059,6 +1110,7 @@ module Sequel
 
     # Add a common table expression (CTE) with the given name and a dataset that defines the CTE.
     # A common table expression acts as an inline view for the query.
+    #
     # Options:
     # :args :: Specify the arguments/columns for the CTE, should be an array of symbols.
     # :recursive :: Specify that this is a recursive CTE
@@ -1079,10 +1131,35 @@ module Sequel
 
     # Add a recursive common table expression (CTE) with the given name, a dataset that
     # defines the nonrecursive part of the CTE, and a dataset that defines the recursive part
-    # of the CTE.  Options:
+    # of the CTE.
+    #
+    # Options:
     # :args :: Specify the arguments/columns for the CTE, should be an array of symbols.
     # :union_all :: Set to false to use UNION instead of UNION ALL combining the nonrecursive and recursive parts.
     #
+    # PostgreSQL 14+ Options:
+    # :cycle :: Stop recursive searching when a cycle is detected. Includes two columns in the
+    #           result of the CTE, a cycle column indicating whether a cycle was detected for
+    #           the current row, and a path column for the path traversed to get to the current
+    #           row.  If given, must be a hash with the following keys:
+    #           :columns :: (required) The column or array of columns to use to detect a cycle.
+    #                       If the value of these columns match columns already traversed, then
+    #                       a cycle is detected, and recursive searching will not traverse beyond
+    #                       the cycle (the CTE will include the row where the cycle was detected).
+    #           :cycle_column :: The name of the cycle column in the output, defaults to :is_cycle.
+    #           :cycle_value :: The value of the cycle column in the output if the current row was
+    #                           detected as a cycle, defaults to true.
+    #           :noncycle_value :: The value of the cycle column in the output if the current row
+    #                              was not detected as a cycle, defaults to false. Only respected
+    #                              if :cycle_value is given.
+    #           :path_column :: The name of the path column in the output, defaults to :path.
+    # :search :: Include an order column in the result of the CTE that allows for breadth or
+    #            depth first searching. If given, must be a hash with the following keys:
+    #            :by :: (required) The column or array of columns to search by.
+    #            :order_column :: The name of the order column in the output, defaults to :ordercol.
+    #            :type :: Set to :breadth to use breadth-first searching (depth-first searching
+    #                     is the default).
+    #
     #   DB[:t].with_recursive(:t,
     #     DB[:i1].select(:id, :parent_id).where(parent_id: nil),
     #     DB[:i1].join(:t, id: :parent_id).select(Sequel[:i1][:id], Sequel[:i1][:parent_id]),
@@ -1093,6 +1170,21 @@ module Sequel
     #   #   UNION ALL
     #   #   SELECT i1.id, i1.parent_id FROM i1 INNER JOIN t ON (t.id = i1.parent_id)
     #   # ) SELECT * FROM t
+    #
+    #   DB[:t].with_recursive(:t,
+    #     DB[:i1].where(parent_id: nil),
+    #     DB[:i1].join(:t, id: :parent_id).select_all(:i1),
+    #     search: {by: :id, type: :breadth},
+    #     cycle: {columns: :id, cycle_value: 1, noncycle_value: 2})
+    #     
+    #   # WITH RECURSIVE t AS (
+    #   #     SELECT * FROM i1 WHERE (parent_id IS NULL)
+    #   #     UNION ALL
+    #   #     (SELECT i1.* FROM i1 INNER JOIN t ON (t.id = i1.parent_id))
+    #   #   )
+    #   #   SEARCH BREADTH FIRST BY id SET ordercol
+    #   #   CYCLE id SET is_cycle TO 1 DEFAULT 2 USING path
+    #   # SELECT * FROM t
     def with_recursive(name, nonrecursive, recursive, opts=OPTS)
       raise(Error, 'This dataset does not support common table expressions') unless supports_cte?
       if hoist_cte?(nonrecursive)
@@ -1245,6 +1337,18 @@ module Sequel
       end
     end
 
+    # Append to the current MERGE WHEN clauses.
+    # Mutates the hash to add the conditions, if a virtual row block is passed.
+    def _merge_when(hash, &block)
+      hash[:conditions] = Sequel.virtual_row(&block) if block
+
+      if merge_when = @opts[:merge_when]
+        clone(:merge_when => (merge_when.dup << hash.freeze).freeze)
+      else
+        clone(:merge_when => [hash.freeze].freeze)
+      end
+    end
+
     # Add the given filter condition. Arguments:
     # clause :: Symbol or which SQL clause to effect, should be :where or :having
     # cond :: The filter condition to add
